=begin
  
Developed by Brian Concannon
Copyright abandoned 2012 Brian Concannon, placed in the public domain by Brian Concannon 

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Name :           Debuggifier.rb 1.00
Description :   Provides breakpoints, watches, traces, and other debugging functionality to
                a Sketchup plugin, for Ruby developers
Author :        Brian Concannon, brian_concannon@yahoo.com
Usage :         1. Install into the plugins directory (or subfolder - supports organizer.rb) 
                2. Provide debugging statements within your plugin's code using Debuggifier.exe
                3. Run Sketchup to use your plugin.
Date :      2012-01-15
History:
    1.00    (2011-10-01)    Initial version
	2.03    (2012-02-05)    Initial release
	2.04    (2012-02-06)    Bug fix-dfrMyPuts did not exist
	                          Also put comments at start of each method in this code
Note:  A user guide in pdf format is available
=end
=begin
Variable name prefixes to show INTENDED usage of each variable:
(often Ruby processing does not agree, however)

  fi_  file
  h_   hash
  i_   integer scalar         is_  set of integer scalars
  s_   floating point scalar  ss_  set of floating point scalars
  t_   text                   ts_  set of texts
  u_   undetermined type

to do:
  - Show next line of code, not previous line of code.
  
=end

require 'Win32API'
#----------------------------------------------------------------------------
class Debuggifier
    @i_initialized=0
    @i_initializationattempts=0
    @i_print_nth=1000
    @i_printnthlim=1
    @i_printlinecount=0
    @i_printed=0

    FAILED=1          #returned constants
    SUCCESS=0
    MANDATORY=0      #get setup constants
    OPTIONAL=1
    COND_NUM_CHANGED=0   #conditional watch constants
    COND_NUM_ASSIGNED=1
    COND_NUM_GT=2
    COND_NUM_LT=3
    COND_NUM_EQ=4
    COND_NUM_NE=5
    COND_TXT_CHANGED=6
    COND_TXT_ASSIGNED=7
    COND_TXT_GT=8
    COND_TXT_LT=9
    COND_TXT_EQ=10
    COND_TXT_NE=11    
    COND_UNKNOWN=12
    IA_ONESTEP=1      #interaction constants
    IA_NOSTEP=2
    IA_GO=3
    #IA_SKIP=4
    IA_ABORT=5
    YES=1          #...
    NO=0
#----------------------------------------------------------------------------
def Debuggifier.dfrInitIfNecessary
begin
    Sketchup.send_action "showRubyPanel:"
    @i_CodeLineCount=0
    @i_printed=0
    @i_ConditionExprType=-1
    @i_LineLast=10
    @i_LineFirst=1
    @i_InteractMode=IA_ONESTEP
    @ts_CodeLines=[]
    @ts_condcode=[]
    @ts_condcode.push 'CHANGED'
    @ts_condcode.push 'ASSIGNED'
    @ts_condcode.push '>'
    @ts_condcode.push '<'
    @ts_condcode.push '='
    @ts_condcode.push '!='
    @ts_condcode.push 'TEXT CHANGED'
    @ts_condcode.push 'TEXT ASSIGNED'
    @ts_condcode.push 'TEXT >'
    @ts_condcode.push 'TEXT <'
    @ts_condcode.push 'TEXT ='
    @ts_condcode.push 'TEXT !='
    @ts_condcode.push '?'
    @h_WatchValueHash = {}
    @h_WatchCodeHash = {}
    @h_BreakpointHash = {} 
    @h_VariableValueHash = {}
    @t_SettingsFile='DEBUGGIFIER_SETUP.TXT'
    @t_DebuggingLogFile='DEBUGGIFIER.TXT'
    @t_previous_call='NONE'
    @fi_debugoutputfile = File.new(@t_DebuggingLogFile, "w")
    r = Debuggifier.dfrGetSettings(OPTIONAL)
    if r==FAILED
      return FAILED
    end  
    r = Debuggifier.dfrGetCodeLines
    if r==FAILED
      return FAILED
    end  
    @i_initialized=1

  return SUCCESS
rescue 
  msg='Rescue from dfrInitIfNecessary'
  UI.messagebox(msg)
  @i_InteractMode=IA_ABORT     #no more trying repeatedly ...
  return FAILED
end
end    
#----------------------------------------------------------------------------
def Debuggifier.dfrMyPuts(t_text)  #version 2.04 bug fix, was dfrMyPrint
#
# trapping point for development
#
#nodebugplease
  puts t_text
end
#----------------------------------------------------------------------------
def Debuggifier.dfrMyPrint(*u_args)
#
# This is the method that handles an incoming debug call from the user's plugin
#
#nodebugplease
begin
  if @i_InteractMode==IA_ABORT
    return FAILED
  end
  @t_previous_call=u_args
  @t_string0=u_args[0]
  i_result=Debuggifier.dfrSetupMyprint
  if i_result==FAILED
    return FAILED
  end 
  @i_t_Len=u_args.length
  i_result=Debuggifier.dfrCheckPrms
  if i_result==FAILED
    return FAILED
  end 

  if @i_t_Len==1  #1 arg means a simple oneline print, or a break request
    i_result=Debuggifier.dfrMyPrintOneArgument
    if i_result==FAILED
      return FAILED
    end 
    i_result=Debuggifier.dfrProcessBreakpoint
  elsif @i_t_Len==3  #3 args always means a dfrMyPrint call (string,number,string)
    @t_param0=@t_string0.strip
    @s_param1=u_args[1]    
    @t_param2=u_args[2].strip    
    i_result=Debuggifier.dfrMyPrint3Arguments
    if i_result==FAILED
      return FAILED
    end 
    i_result=Debuggifier.dfrStoreOrWatchOrBreak
    if i_result==FAILED
      return FAILED
    end 
  else
    UI.messagebox('ILLEGAL CALL TO Debuggifier.dfrMyPrint - NUMBER OF PARAMTERS IS '+@i_t_Len.to_s)
    UI.messagebox('OFFENDING CALL OCCURRED AFTER '+@t_previous_call)
    @i_InteractMode=IA_ABORT 
  end    
  return SUCCESS
rescue 
  msg='Rescue from dfrMyPrint'
  UI.messagebox(msg)
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrStoreOrWatchOrBreak
#
# This method scans the user's debug call for a watchpoint or breakpoint or variable
#
begin
    if @i_InteractMode!=IA_ABORT
      r=Debuggifier.dfrFindAndShowWatch
      if r==FAILED
        return r
      end
      r=Debuggifier.dfrFindAndStoreVariableValue
      if r==FAILED
        return r
      end
      if @i_InteractMode==IA_ONESTEP
        i_result=self.dfrBreak('DEBUGGING PAUSE')
      end
    end
    return SUCCESS
rescue 
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrCheckPrms
#
# This method screens out debug calls with no arguments
#
begin
  if @i_t_Len==0
    UI.messagebox('ILLEGAL CALL TO Debuggifier.dfrMyPrint - NO PARAMETERS PROVIDED')
    UI.messagebox('OFFENDING CALL OCCURRED AFTER '+@t_previous_call)
    @i_InteractMode=IA_ABORT
    return FAILED    
  end
  return SUCCESS
rescue 
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrMyPrint3Arguments
#
# This method handles debug calls with 3 arguments, which are LHS or RHS evaluations
#
begin
    @t_s_param1=@s_param1.to_s
    if @t_s_param1==''
      @t_s_param1='nil'
    end                                                                                          
    @i_printlinecount+=1
    t_prnlc=Debuggifier.dfrLeftPad(@i_printlinecount)
    t_prncln=Debuggifier.dfrLeftPad(@i_codelinenum)
    t_all=t_prnlc +' ' + t_prncln + ' ' + @t_param0 + ' ' + @t_param2 + '=' + @t_s_param1 +       "\n"
    if @i_printlinecount>=@i_print_nth
      @fi_debugoutputfile.syswrite(t_all)
    else 
      @i_printlinecount=0
    end
    return SUCCESS
rescue 
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrRightPad(t_text)
#
# This method pads a string with spaces on its end
#
begin
  t_string=t_text.to_s
  @i_t_Len=t_string.length
  if @i_t_Len>20
    return SUCCESS
  end
  while @i_t_Len<20
    t_string+=' '
    @i_t_Len=t_string.length
  end
  return t_string
rescue 
  return t_string
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrLeftPad(i_int)
#
# This method pads a string with spaces on its start
#
begin
  t_string=i_int.to_s
  @i_t_Len=t_string.length
  if @i_t_Len>6
    return SUCCESS
  end
  while @i_t_Len<7
    t_string=' '+t_string
    @i_t_Len=t_string.length
  end
  return t_string
rescue 
  return t_string
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrMyPrintOneArgument
#
# This method handles debug calls with only one argument, which is the text of a code line
#
begin
    @t_string0=@t_string0.to_s.strip
    @i_printlinecount+=1
    if @i_printlinecount>=@i_print_nth
      t_line=self.dfrLeftPad(@i_printed)
      tt_= t_line+'    '+@t_string0 + "\n"
      @fi_debugoutputfile.syswrite(tt_)
    else 
      @i_printlinecount=0
      tt_="code not implemented for @i_print_nth>1"
      UI.messagebox(tt_)
      return FAILED
    end
    ts_words=@t_string0.scan(/\w+/)
    @i_codelinenum=ts_words[0].to_i
    return SUCCESS
rescue 
  msg='Rescue from dfrMyPrintOneArgument'
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrSetupMyprint
#
# This method handles some initialization tasks
#
begin
  if @i_initializationattempts>2
    return SUCCESS   
  end
  if @i_InteractMode==IA_ABORT
    return SUCCESS
  end
  if @i_initialized==0
    @i_initializationattempts+=1
    r=Debuggifier.dfrInitIfNecessary
    if r==FAILED 
      return FAILED
    end
  end
  @i_printed+=1
  if @i_printed>=@i_printnthlim
    @i_print_nth=1
  end
  return SUCCESS
rescue 
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrCheckPrintInterval(ts_words)
#
#this feature is no longer used
#
  return SUCCESS
end
#----------------------------------------------------------------------------
def Debuggifier.dfrLogEachLine
#
#this feature is no longer used
#
@i_printnthlim=1
end
#----------------------------------------------------------------------------
def Debuggifier.dfrBreak(u_args)
#
# This method handles breakpoint logic after it is determined that this debug
# call does indeed require a breakpoint
#
begin
  if @i_InteractMode==IA_ABORT
    return FAILED
  end
  if @i_initialized==0
    r=Debuggifier.dfrInitIfNecessary
    if r==FAILED 
      return FAILED
    end
  end
  @t_previous_call=u_args
#
#   interact with user
#
  i_loops=0
  @i_InteractMode=IA_NOSTEP
  while @i_InteractMode==IA_NOSTEP
    i_return=self.dfrInteractViaInputbox
    i_loops+=1
    if i_loops>1000
      @i_InteractMode=IA_NOSTEP  #prevents infinite loop.  That happenned once.
    end
    if i_return==FAILED
      if @i_InteractMode!=IA_ABORT
        @i_InteractMode=IA_NOSTEP
      else
        return FAILED
      end
    end
  end
  return i_return
rescue
 Debuggifier.dfrMyPrint 'rescue from Debuggifier.dfrBreak'
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrInteractViaInputbox
#
# This method handles interactions with the user via input boxes
# 
begin
  t_action='STEP'
  while @i_InteractMode==IA_NOSTEP do
    t_bp=@h_BreakpointHash.length.to_s
    t_w=@h_WatchValueHash.length.to_s
    t_v=@h_VariableValueHash.length.to_s
    ts_prompts=["Actions",t_bp+" Breakpoints",t_w+" Watches",t_v+" Variables"]
    ts_defaults=[t_action,"","",""]
    t_action="STEP|GO|ABORT|LIST|SAVE|LOAD"
    t_breakpoints="SHOW ALL|CLEAR ALL|ADD|DELETE"
    t_watches=    "SHOW ALL|CLEAR ALL|ADD|DELETE"
    t_values=     "SHOW ALL|EXAMINE|SET"
    ts_dropdowns=[t_action,t_breakpoints,t_watches,t_values]
    ts_Inputs=UI.inputbox(ts_prompts,ts_defaults,ts_dropdowns,'Debuggifier')
    #if ts_Inputs==nil  why did this fail?    -btc
#      return FAILED
#    end
    t_actionsbox=ts_Inputs[0]
    t_actionsbox=t_actionsbox.strip
    if t_actionsbox==""
      t_actionsbox="ABORT"
    end
    if t_actionsbox==nil
      t_actionsbox="ABORT"
    end
    if t_actionsbox=="ABORT"
      @i_InteractMode=IA_ABORT
      return FAILED
    end
    t_breakpointsbox=ts_Inputs[1].strip
    t_watchbox=ts_Inputs[2].strip
    t_values=ts_Inputs[3].strip
    okb=dfrProcessBoxString(t_breakpointsbox,'Breakpoint',@h_BreakpointHash)
    okw=dfrProcessBoxString(t_watchbox,    'Watch',     @h_WatchValueHash)
    okv=dfrProcessBoxString(t_values,      'Value',     @h_VariableValueHash)
    oka=dfrProcessActions(t_actionsbox)
    if oka==FAILED
      return FAILED
    end
    if okw==FAILED
      if t_watchbox!=''
        msg='Syntax error in watch command ='+t_watchbox
        UI.messagebox(msg)  
        return FAILED
      end            
    end
    if okb==FAILED
      if t_breakpointsbox!=''
        msg='Syntax error in breakpoint command ='+t_breakpointsbox
        UI.messagebox(msg)  
        return FAILED
      end            
    end
    if okv==FAILED
      msg='Problem in logic for <show all values>'
      UI.messagebox(msg)  
      @i_InteractMode=IA_ABORT
      return FAILED
    end
  end  
  return SUCCESS
rescue
  return FAILED
end
end  
#----------------------------------------------------------------------------
def Debuggifier.dfrProcessActions(t_actionsbox)
#
# This method handles the user's entry in the Actions line of the main inputbox
#
begin
    t_actionsbox=t_actionsbox.upcase.strip
    if t_actionsbox=="STEP"
      @i_InteractMode=IA_ONESTEP
      ok=SUCCESS
    elsif t_actionsbox=="GO"
        @i_InteractMode=IA_GO
        ok=SUCCESS
    elsif t_actionsbox=="ABORT"
      @i_InteractMode=IA_ABORT
        ok=SUCCESS
    elsif t_actionsbox=="SAVE"
      ok=Debuggifier.dfrSaveSettings
      @i_InteractMode=IA_NOSTEP
    elsif t_actionsbox=="LOAD"
      ok = self.dfrGetSettings(MANDATORY)
      @i_InteractMode=IA_NOSTEP
    elsif t_actionsbox=="LIST"
      @i_InteractMode=IA_NOSTEP
      @i_LineLast=@i_codelinenum+5
      if @i_LineLast>@i_CodeLineCount
        @i_LineLast=@i_CodeLineCount
        @i_LineFirst=@i_CodeLineCount-10
      else
        @i_LineFirst=@i_codelinenum-5
        if @i_LineFirst<0
          @i_LineFirst=1
        end
      end
      i_n1=@i_LineFirst
      i_n2=@i_LineLast
      Debuggifier.dfrListProgramCodeLines(i_n1,i_n2)
      ok=SUCCESS
    end
    return ok
rescue
    @i_InteractMode=IA_ONESTEP
    return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrUserSet
#
# This method gets, then handles the user's entries into the secondary inputbox for
# setting the value of a variable
#
begin
      @i_InteractMode=IA_NOSTEP
      i_return=self.dfrGetVariableNameFromUser
      if i_return==FAILED
        return FAILED
      end
      i_return=self.dfrGetVariableValueFromUser
      if i_return==FAILED
        return FAILED
      end
      @s_test=@s_requestedvalue
      t_eval=@t_requestedvarname+'=@s_test'
begin 
      eval t_eval   #sometimes fails
rescue
      @s_test=nil
end 
      t_valuestring=@s_test.to_s
      i_return=self.dfrShowVarValue2Text(t_eval)
      if i_return==FAILED
        return FAILED
      end
      @h_VariableValueHash.store(@t_requestedvarname,@s_test)      
      UI.messagebox(@t_requestedvarname+"=<"+t_valuestring+">")
    return SUCCESS
rescue
    @i_InteractMode=IA_ONESTEP
    return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrExamine
#
# This method gets, then handles the user's entries into the secondary inputbox for
# examining the value of a variable
#
begin
  i_return=self.dfrGetVariableNameFromUser
  @i_InteractMode=IA_NOSTEP
  if i_return==FAILED
    return FAILED
  end
  @u_test=''
  t_eval='@u_test='+@t_requestedvarname
  @i_evalfailed=0
begin 
  eval t_eval   #quite often fails, eval is flakey
rescue
  @u_test=nil
  @i_evalfailed=1
end 
  @u_hash_data=@h_VariableValueHash[@t_requestedvarname]
  i_return=self.dfrShowVarText2Value(t_eval)
  if i_return==FAILED
    return FAILED
  end
  @i_InteractMode=IA_NOSTEP
  return SUCCESS
rescue
  @i_InteractMode=IA_ONESTEP
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrShowValuesToMsgbox
#
# This method shows a messagebox listing names and values of all known variables
#
begin
  i_return=self.dfrMakeMultilineHashesString('Value')
  i_return=self.dfrShowHashMsgbox(@h_VariableValueHash,'Value')    
  return SUCCESS
rescue
    return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrGetVariableNameFromUser
#
# This method gets, then handles the user's entries into a secondary inputbox
# for a variable name
#
begin
      ts_prompts=["Variable name"]
      ts_defaults=[""]
      ts_dropdowns=[""]
      t_varname=UI.inputbox(ts_prompts,ts_defaults,ts_dropdowns,"Examine/Set")
#     if t_varname==nil
#       return FAILED
#     end
      i_vnlength=t_varname.length
      if i_vnlength>1
        ts_words=t_varname.scan(/\w+/)
          t_varname=ts_words[0].strip
      end
      @t_requestedvarname=t_varname.to_s
      return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrGetVariableValueFromUser
#
# This method gets, then handles the user's entries into a secondary inputbox
# for a value (numeric values are default type, text values start with ')
begin 
      ts_prompts=["New value"]
      ts_defaults=[""]
      ts_dropdowns=[""]
      t_varvalue=UI.inputbox(ts_prompts,ts_defaults,ts_dropdowns,"Value")
#     if t_varvalue==nil
#       return FAILED    why did this cause an error?
#     end
      i_vnlength=t_varvalue.length
      if i_vnlength>1
        ts_words=t_varvalue.scan(/\w+/)
        t_varvalue=ts_words[0].strip
      end
      t_varvalue=t_varvalue.to_s
      @s_requestedvalue=t_varvalue.to_f
      return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrShowVarText2Value(t_eval)
#
# This method converts value of a known variable name (of unknown type) if 
# the eval function failed
#
begin
    t_hash_data=@u_test.to_s
    if @i_evalfailed==1
      if defined? @t_requestedvarname
        if @u_hash_data==nil
          t_hash_data='nil'
        else
          t_hash_data=@u_hash_data.to_s
          if t_hash_data.length==0
            t_hash_data='null string, nil, or empty'
          end
        end
      end
    end
    t_output=@t_requestedvarname+"=<"+t_hash_data+">"
    UI.messagebox(t_output)
    return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrShowVarValue2Text(t_eval)
#
# This method shows a variable's value in a messagebox
#
begin
     if @s_test==nil
        UI.messagebox(@t_requestedvarname+" not found")      
        @i_InteractMode=IA_ABORT
      else
        t_output=@t_requestedvarname+"=<"+@s_test.to_s+">"
        UI.messagebox(t_output)      
      end
      return SUCCESS 
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrProcessBoxString(t_boxstring,t_boxtype, h_thehash)
#
# This method handles user's entries into the Breakpoints, Watches, and 
# Variables textboxes of the main inputbox
#
begin
  t_words1=t_boxstring.strip
  if t_boxstring.length<=0
    return SUCCESS
  end
  ok=0
  if t_boxstring.upcase=="SHOW ALL"
    if t_boxtype=='Value'
      if t_boxstring!=nil
        ok+=dfrShowValuesToMsgbox
      end
    else
      ok+=self.dfrShowHashMsgbox(h_thehash,t_boxtype)    
    end
  elsif t_boxstring.upcase=="CLEAR ALL"
    h_thehash={}  #why does this fail to clear the hash?
    if t_boxtype=='Watch'
      @h_WatchValueHash = {}
      @h_WatchCodeHash = {}
    end
    if t_boxtype=='Breakpoint'
      @h_BreakpointHash = {} 
    end
    if t_boxtype=='Value'
      @h_VariableValueHash = {}
    end
    ok+=self.dfrShowHashMsgbox(h_thehash,t_boxtype)
  else
    ts_words=t_words1.scan(/\w+/)
    t_boxstringlength=ts_words.length.to_s
    if t_boxstringlength=='1'
      t_w0=ts_words[0].strip
      if t_w0=="ADD"
        #adding breakpoint
        if t_boxtype=="Breakpoint"
          ok+=self.dfrAddBreakpoint
        end
        #adding watch
        if t_boxtype=='Watch'
          ok+=self.dfrAddWatch
        end
      end
      if t_w0=='DELETE'
        if t_boxtype=="Breakpoint"
        #deleting breakpoint
          ok+=self.dfrDeleteBreakpoint
        end
        if t_boxtype=='Watch'
        #deleting watch
          ok+=self.dfrDeleteWatch
        end
      end  
      if t_w0=='EXAMINE'
        if t_boxtype=="Value"
          ok+=self.dfrExamine
        end
      end
      if t_w0=='SET'
        if t_boxtype=="Value"
          ok+=self.dfrUserSet
        end
      end
    end
  end
  if ok>0
    self.ShowSyntaxError(t_words1)
  end
  return ok
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrDeleteWatch
#
# This method gets, then handles user's entries for deleting a watch
#
begin
  ts_prompts=["Variable name"]
  ts_defaults=[" "]
  ts_dropdowns=[""]
  t_title="Delete watchpoint"
  ts_Inputs=UI.inputbox(ts_prompts,ts_defaults,ts_dropdowns,t_title)
  t_VarName=ts_Inputs[0].strip
  @h_WatchVariableHash.delete t_VarName
  @h_WatchCodeHash.delete t_VarName
  return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrAddWatch
#
# This method gets, then handles user's entries for adding a watch
#
begin
  ts_prompts=["Variable name","Break if","Value"]
  ts_defaults=["","CHANGED",""]
  ts_dropdowns=["","CHANGED|ASSIGNED|>|<|==|!=|",""]
  t_title="Watchpoint details"
  ts_Inputs=UI.inputbox(ts_prompts,ts_defaults,ts_dropdowns,t_title)
  t_VarName=ts_Inputs[0].strip
  t_Condition=ts_Inputs[1].strip
  t_value=ts_Inputs[2].strip
  t_first=t_value[0]
  i_first=t_first.to_i
  if i_first==39  #apostrophe?
    @i_ConditionExprType=self.dfrGetTextualConditionCode(t_Condition)
    i_Len=t_value.length
    t_value=t_value[1,i_Len-1]
    u_value=t_value
  else
    @i_ConditionExprType=self.dfrGetNumericConditionCode(t_Condition)
    u_value=t_value.to_f
  end
  @h_WatchValueHash.store(t_VarName,u_value)
  @h_WatchCodeHash.store(t_VarName,@i_ConditionExprType)
  return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrAddBreakpoint
#
# This method gets, then handles user's entries for addubg a breakpoint
#
begin
  ok=SUCCESS
  ts_prompts=["Method","Line number"]
  ts_defaults=["activate",""]
  ts_dropdowns=["",""]
  t_title="Breakpoint details"
  ts_Inputs=UI.inputbox(ts_prompts,ts_defaults,ts_dropdowns,t_title.to_s)
  t_wordsin = ts_Inputs
  t_inputcount=t_wordsin.length.to_s
  t_input0= t_wordsin[0].to_s
  t_Method=t_wordsin[0].strip
  if t_inputcount=='2'
    t_LineNumber=t_wordsin[1].strip
  else
    t_LineNumber=0
  end
  if t_Method!=""
    if t_LineNumber!=""
      UI.messagebox("Use one or the other.")
      ok+=FAILED
    else
      t_Push="M "+t_Method
       @h_BreakpointHash.store(t_Push,t_Push)
    end
  else
    if t_LineNumber==""
      UI.messagebox("Provide breakpoint.")
      ok+=FAILED
    else
      t_Push="L "+t_LineNumber
      @h_BreakpointHash.store(t_Push,t_Push)
    end
  end
  return ok
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrShowHashMsgbox(h_thehash,t_boxtype)
#
# This method shows the entire contents of the Breakpoints hash, or the Watches hash, or the
# Variables hash in a messagebox
#
begin
    i_Len=h_thehash.length
    if i_Len==0
      @t_show="Empty "+t_boxtype+" list.\n"
      uiresult=UI.messagebox(@t_show)    
      return SUCCESS
    end 
    i_return=self.dfrMakeMultilineHashesString(t_boxtype)
    if i_return!=SUCCESS
      return FAILED
    end
    uiresult=0
    uiresult=UI.messagebox(@t_show,MB_OKCANCEL)
    if uiresult==2  #cancelled?
       uiresult=UI.messagebox('You cancelled.  Abort debugging?',MB_YESNO)
       if uiresult==6
         @i_InteractMode=IA_ABORT
       end
    end
    return SUCCESS
rescue
  self.dfrMyPuts 'rescue from Debuggifier.dfrShowHashMsgbox'
  return FAILED
end
end    
#----------------------------------------------------------------------------
def Debuggifier.dfrGetNumericConditionCode(t_condcode_)
#
# This method converts an operator symbol to an internal constant
#
begin
  t_condcode=t_condcode_.to_s
  if t_condcode=='CHANGED'
   i_cond=COND_NUM_CHANGED
  elsif t_condcode=="ASSIGNED"
    i_cond=COND_NUM_ASSIGNED
  elsif t_condcode==">"
    i_cond=COND_NUM_GT 
  elsif t_condcode=="<"
    i_cond=COND_NUM_LT
  elsif t_condcode=="=="
    i_cond=COND_NUM_EQ
  elsif t_condcode=="="
    i_cond=COND_NUM_EQ
  elsif t_condcode=="!="
    i_cond=COND_NUM_NE
  else
    i_cond=COND_UNKNOWN
    UI.messagebox('Unknown condition: '+t_condcode_)
    @i_InteractMode=IA_ABORT
    return FAILED
  end
  return i_cond
rescue
 Debuggifier.dfrMyPrint 'Rescue from dfrGetNumericConditionCode'
  return COND_UNKNOWN
end    
end
#----------------------------------------------------------------------------
def Debuggifier.dfrGetTextualConditionCode(t_condcode)
#
# This method converts text for an operator to a code
#
begin
  t_condcode=t_condcode.to_s
  if t_condcode=='CHANGED'
    i_cond=COND_TXT_CHANGED
  elsif t_condcode=="ASSIGNED"
    i_cond=COND_TXT_ASSIGNED
  elsif t_condcode==">"
    i_cond=COND_TXT_GT 
  elsif t_condcode=="<"
    i_cond=COND_TXT_LT
  elsif t_condcode=="=="
    i_cond=COND_TXT_EQ
  elsif t_condcode=="="
    i_cond=COND_TXT_EQ
  elsif t_condcode=="!="
    i_cond=COND_TXT_NE
  else
    i_cond=COND_UNKNOWN
    UI.messagebox('Unknown condition; '+t_condcode)
    @i_InteractMode=IA_ABORT
    return FAILED
  end
  @i_ConditionExprType=i_cond
  return i_cond
rescue
 Debuggifier.dfrMyPrint 'Rescue from dfrGetTextualConditionCode'
  return COND_UNKNOWN
end    
end
#----------------------------------------------------------------------------
def Debuggifier.dfrGetConditionDescr(i_condcode)
#
# This method converts a condition code to text
#
begin
  if i_condcode==COND_NUM_CHANGED
    t_condcode="CHANGED"
  elsif i_condcode==COND_NUM_ASSIGNED
    t_condcode="ASSIGNED"
  elsif i_condcode==COND_NUM_GT
    t_condcode=">"
  elsif i_condcode==COND_NUM_LT
    t_condcode="<"
  elsif i_condcode==COND_GE
    t_condcode=">="
  elsif i_condcode==COND_LE
    t_condcode="<="
  elsif i_condcode==COND_NUM_EQ
    t_condcode="="
  elsif i_condcode==COND_NUM_NE
    t_condcode="!="
  else
    t_condcode=COND_UNKNOWN
    UI.messagebox('Unknown condition, '+i_condcode)
    @i_InteractMode=IA_ABORT
  end
  @i_ConditionExprType=t_condcode
  return SUCCESS
rescue
 Debuggifier.dfrMyPrint 'Rescue from GetConditionDescr'
  return FAILED
end    
end
#----------------------------------------------------------------------------
def Debuggifier.ShowSyntaxError(strA)
#
# This method show a syntax error caused by bad input of the user
#
begin
  msg='Syntax error in command  ='+ strA
  UI.messagebox(msg)
  return SUCCESS
rescue    
 Debuggifier.dfrMyPrint 'rescue from Debuggifier.ShowSyntaxError'
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrSaveSettings
#
#  This method saves 'user settings' to the setup file (file locations,
#  breakpoint information, and watch information)
#
begin
    @fi_dfrSettingsFile = File.new(@t_SettingsFile, "w")
#
#   save breakpoints
#
    t_out=dfrRightPad(@h_BreakpointHash.length.to_s)
    t_out+="#breakpoint count\n"
    @fi_dfrSettingsFile.syswrite(t_out)
    @t_show=""
    h_thehash=@h_BreakpointHash
    i_cnt=0
    h_thehash.each do|nam,val|
      i_cnt+=1
      t_out=dfrRightPad(val)
      t_out+="#breakpoint "+i_cnt.to_s+" \n"
      @t_show+=t_out
      
    end
      
    @fi_dfrSettingsFile.syswrite(@t_show)
#
#   save watched variables and conditions
#
    t_out=dfrRightPad(@h_WatchValueHash.length.to_s)
    t_out+="#watch count\n"
    @fi_dfrSettingsFile.syswrite(t_out)
    i_return=self.dfrMakeMultilineHashesString("Watch")
    if i_return!=SUCCESS
      return FAILED
    end
    @fi_dfrSettingsFile.syswrite(@t_show)
#
#   close up
#
    @fi_dfrSettingsFile.close
    return SUCCESS
rescue
  UI.messagebox('Error saving settings to DEBUGGIFIER_SETUP.TXT, please edit and fix it.')
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrMakeMultilineHashesString(t_boxtype)
#
# This method creates a multiline string (uses newline characters) representing
#   a hash.  watchhash has special code in it that needs translation from a 
#   condition type number to an English word for the code
#
begin
  @t_show=""
  if t_boxtype=="Breakpoint"
    i_result=Debuggifier.dfrMakeBreakStrings
    return SUCCESS
  elsif t_boxtype=="Watch"
    i_result=Debuggifier.dfrMakeWatchStrings
    return SUCCESS
  elsif t_boxtype=="Value"
    i_result=Debuggifier.dfrMakeValueStrings
    return SUCCESS
  end
#
# cleanup
#
  UI.messagebox('ILLEGAL CALL TO DEBUGGIFIER.dfrMakeMultilineHashesString')
  @i_InteractMode=IA_ABORT
  return FAILED
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrMakeBreakStrings  
#
# This method composes textual output of breakpoint information
#
begin
    ts_array=[]
    h_thehash=@h_BreakpointHash
    i_cnt=0
    t_hash=h_thehash.to_s
    h_thehash.each do|nam,val|
      t_valstring=val.to_s
      if t_valstring.length==0
        UI.messagebox('zero length breakpoint hash entry')
        @i_InteractMode=IA_ABORT
        return SUCCESS
      end
      ts_words=val.scan(/\w+/)
      t_LorM=ts_words[0].strip
      t_MethodOrLine=t_LorM.to_s
      i_cnt+=1
      if t_LorM=="L"
        t_LorM="LINE "
      end
      if t_LorM=="M"
        t_LorM="METHOD "
      end
      t_where=ts_words[1].to_s.strip
      t_add=t_LorM +' '+ t_where
      t_add=self.dfrRightPad(t_add) 
      t_add+= '#breakpoint ' + i_cnt.to_s + "  \n"
      ts_array.push t_add 
    end
    ts_sorted=ts_array.sort
    ts_sorted.each do |t_data|
      @t_show+=t_data
    end
    return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrMakeWatchStrings  
#
# This method composes textual output of watch information
#
begin
    ts_array=[]
    h_thehash=@h_WatchValueHash
    i_cnt=0
    t_hash=h_thehash.to_s
    h_thehash.each do|nam,val|
      if val.to_s==nil                         
      end
      if val.to_s==""                         
      end         
      t_valstring=val.to_s
      if t_valstring.length==0
        UI.messagebox('zero length watch hash entry')
        @i_InteractMode=IA_ABORT
        return SUCCESS
      end
      t_namstring=nam.to_s
      i_val=@h_WatchCodeHash[t_namstring]
      t_translatedcode=@ts_condcode[i_val].to_s
      @s_test=@h_WatchValueHash[t_namstring]
      t_val=@s_test.to_s
      if t_translatedcode[0,5]=='TEXT '
        t_val='\''+t_val
      end
      i_cnt+=1
      if i_val==COND_NUM_ASSIGNED
        t_val=''
      elsif i_val==COND_NUM_CHANGED
        t_val=''
      elsif i_val==COND_TXT_ASSIGNED
        t_val=''
      elsif i_val==COND_TXT_CHANGED
        t_val=''
      end
      t_add=t_namstring+' '+t_translatedcode + ' ' + t_val
      t_add=self.dfrRightPad(t_add)
      t_add+='#watch ' + i_cnt.to_s+"  \n"

      ts_array.push t_add 
    end
    ts_sorted=ts_array.sort
    ts_sorted.each do |t_data|
      @t_show+=t_data
    end
    return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrMakeValueStrings  
#
# This method composes textual output of information about variables
#
begin
    ts_array=[]
    h_thehash=@h_VariableValueHash
    t_hash=h_thehash.to_s

    h_thehash.each do|nam,val|
      t_namstring=nam.to_s
      if val.to_s==nil
        t_valstring='nil'
      elsif val.to_s==""
        t_valstring='null string, nil, or empty'
      else
        t_valstring=val.to_s
      end         
      t_line=t_namstring+' '+t_valstring
      t_add=t_line+" \n"
      ts_array.push t_add 
    end
    ts_sorted=ts_array.sort
    ts_sorted.each do |t_data|
      @t_show+=t_data
    end
    return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.parse_caller(at)
#
# This method is still experimental
#
    if /^(.+?):(\d+)(?::in `(.*)')?/ =~ at
        file = Regexp.last_match[1]
    end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrGetCodeLines  
#
# This method reads the user's plugin code so it can be listed
#
begin
  if @i_initialized==1
    return SUCCESS
  end
  @t_CodeFileName=__FILE__
  i_PositionPlugins=(__FILE__=~/Plugins/)
  t_path=__FILE__[0,i_PositionPlugins+8]
  @t_CodeFileName=''
  t_caller=caller.last.to_s
  t_save=t_caller
  if t_caller=~/.rb/
    i_Position2= (t_caller=~/.rb/)+3
    t_caller=t_caller[0,i_Position2]
    i_Position1=(t_caller=~/Plugins/)
    if i_Position1>0
      t_caller=t_caller[i_Position1+8,999]
      @t_CodeFileName=t_path+t_caller
    end
  end
  fn=@t_CodeFileName
  if fn==''
    UI.messagebox('Debuggifier could not find the Plugin .rb file to be debugged ' + t_save)
    @i_InteractMode=IA_ABORT
    return FAILED
  end
  @fi_dfrCodeFile = File.open(fn, "r")
  @ts_CodeLines = IO.readlines(fn)
  @fi_dfrCodeFile.close
  @i_CodeLineCount=@ts_CodeLines.length
  return SUCCESS
rescue
  UI.messagebox('FILE ERROR 2' + @t_CodeFileName)  
  @i_InteractMode=IA_ABORT
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrListProgramCodeLines(i_line1,i_line2)
#
# This method sends ten code lines to the Ruby console
#
begin
  if i_line2==0
    i_n2=@i_CodeLineCount-1
  else
    i_n2=i_line2-1
  end
  i_idx=i_line1-1
 
  while i_idx<i_n2
    i_idx+=1
    t_aline=i_idx.to_s+' '+@ts_CodeLines[i_idx]
  end
  return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrFindAndShowWatch
#
# This method scans the debug call to see if any watchpoint criteria are met, and
# causes a break if so.
#
begin
  if @t_param0!='EQ LHS'
    return SUCCESS
  end
  if @t_param2 =~ /\.new/    
                        #look for '.new' so we don't create unneeded stuff
    return SUCCESS
  end
  i_condcode=@h_WatchCodeHash[@t_param2]
  if i_condcode==nil
    return SUCCESS
  end
  u_crit=@h_WatchValueHash[@t_param2]
  @u_test=1
  t_eval='@u_test='+@t_param2
  t_def=defined? @t_param2
#begin 
#  eval '@u_test=t'
#rescue
#  @u_test=nil
#end 
  @u_test=@s_param1  
#  if @u_test==nil
#    return SUCCESS
#  end
  if i_condcode==COND_NUM_CHANGED
    u_valb4=nil
    i_noval=1
  elsif i_condcode==COND_TXT_CHANGED
    u_valb4=nil
    i_noval=1
  elsif i_condcode==COND_NUM_CHANGED
    u_valb4=nil
    i_noval=1
  elsif i_condcode==COND_TXT_ASSIGNED
    u_valb4=nil
    i_noval=1
  else
    u_valb4=@h_VariableValueHash[@t_param2]
    i_noval=0
  end
  if self.dfrTestValue(@u_test,u_crit,i_condcode,u_valb4)==YES
    if i_noval==1
      msg='Conditionally watched variable ' + @t_param2 + " changed, was= " + u_valb4.to_s+"\n"
      msg+='Conditionally watched variable ' + @t_param2 + " changed, now= " + @u_test.to_s
    else
      msg='Conditionally watched variable ' + @t_param2 + " = " + @u_test.to_s
    end
    Debuggifier.dfrMyPuts msg
    UI.messagebox(msg)
    @i_InteractMode=IA_ONESTEP
  end
  return SUCCESS
rescue
 Debuggifier.dfrMyPrint 'Rescue from dfrFindAndShowWatch'
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrFindAndStoreVariableValue
#
# This method scans the debug call to see if any variable's info can be extracted
#
begin
#for example,
#@t_param0 <EQ LHS>
#@t_param2 <@ip>
#@t_s_param1 <#<Sketchup::InputPoint:0xfe5b578>>  
  @t_vvkey=nil
  @u_vvdata=nil
  if @t_param0!='EQ LHS'
    return SUCCESS
  end
  if @t_param2 =~ /\.new/    
                        #look for '.new' so we don't create unneeded stuff
    return SUCCESS
  end
  @t_vvkey=@t_param2.strip
  @u_vvdata=@s_param1
  @h_VariableValueHash.store(@t_vvkey,@u_vvdata)
                                 #i_return=self.dfrShowHashMsgbox(@h_VariableValueHash,'Value')    
  return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrTestValue(s_val,s_criterion,i_code,s_valb4)
#
# This method is where conditional watches are evaluated for true/false result
#
begin
  if i_code==COND_NUM_CHANGED 
    if s_val!=s_valb4
      return YES
    else
      return NO
    end
  elsif i_code==COND_NUM_ASSIGNED
    return YES
  elsif i_code==COND_NUM_GT
    if s_val>s_criterion
      return YES
    else
      return NO
    end
  elsif i_code==COND_NUM_LT
    if s_val<s_criterion
      return YES
    else
      return NO
    end
  elsif i_code==COND_NUM_EQ
    if s_val==s_criterion
      return YES
    else
      return NO
    end
  elsif i_code==COND_NUM_NE
    if s_val!=s_criterion
      return YES
    else
      return NO
    end
  end
  return SUCCESS
rescue
 Debuggifier.dfrMyPrint 'Rescue from dfrTestvalue'
  return FAILED
end
end
#----------------------------------------------------------------------------
#def Debuggifier.dfrShowHashOnRubyConsole(h_Hash,t_descr)  DOES NOT WORK RIGHT YET
#begin
#  return SUCCESS
#rescue
#  return FAILED
#end
#end
#----------------------------------------------------------------------------
def Debuggifier.dfrProcessBreakpoint
#
#   This method determines if there is a breakpoint set at the current line/method, and
#   calls the interaction routine if so.
#
begin
  if not @h_BreakpointHash.empty?
    if @t_string0 =~ / def /
      i_Position= (@t_string0 =~ / def /)
      if i_Position>0
        i_Len=@t_string0.length
        t_method=@t_string0[i_Position+5,i_Len-i_Position-5]
        t_lookfor="M "+t_method
        u_inhash=@h_BreakpointHash[t_lookfor]   #use a fast hash lookup instead of a slow list implementation!
        if not u_inhash.nil?
          msg='Encountered breakpoint at method ' + @t_string0
          @i_InteractMode=IA_NOSTEP
          UI.messagebox(msg)
          r=Debuggifier.dfrBreak('break at method ' + @t_string0)
        end
      end
    else
      t_lookfor="L "+@i_codelinenum.to_s
      u_inhash=@h_BreakpointHash[t_lookfor]   #use a fast hash lookup instead of a slow list implementation!
      if not u_inhash.nil?
        msg='Encountered breakpoint at line ' + @i_codelinenum.to_s
        @i_InteractMode=IA_NOSTEP
        UI.messagebox(msg)
        r=Debuggifier.dfrBreak('break at line ' + @i_codelinenum.to_s)
      end
    end
  end
rescue
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrGetSettings(i_mand)
#
#  This method reads the setup file and fills the hashes for breakpoints and
#  watches
#
begin
  if @i_initialized==1
    if i_mand==OPTIONAL
      return SUCCESS
    end
  end
  fn=@t_SettingsFile
  @fi_dfrSettingsFile=File.open(fn, "r")
#
# read the text lines
#
  ts_arrayraw = IO.readlines(fn)
#
# housekeeping
#
  @fi_dfrSettingsFile.close
  if ts_arrayraw.length==0
    return SUCCESS
  end
#
# remove blank lines
#  
  @ts_setup=[]
  ts_arrayraw.each { |line|
    t_line=line.to_s.strip
    if t_line.length>0
      @ts_setup.push t_line
    end
  }
# Breakpoints
  i_result=self.dfrGetBreakpointsFromSetupFile
#  if i_result==FAILED
#    return FAILED
#  end
# Watches
  i_result=self.dfrGetWatchesFromSetupFile
  if i_result==FAILED
    return FAILED
  end
  return SUCCESS
rescue
  if @i_initializationattempts==0
    UI.messagebox('Error reading DEBUGGIFIER_SETUP.TXT, please edit and fix it.')
  end
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrGetBreakpointsFromSetupFile
#
# This method gets breakpoint information from the setup file
#
#begin                      why is rescue ok without begin and end?
#
# stash count of breakpoints
#
  @i_idx=0
  t_textA=@ts_setup[@i_idx]
  @i_idx+=1
  i_breakpointcount=t_textA.to_i
#
#  fill hash for breakpoints
#
  if i_breakpointcount>0
    i_bpindex=0
    @h_BreakpointHash={}
    while i_bpindex<i_breakpointcount
      t_textA=@ts_setup[@i_idx].strip
      t_textB=Debuggifier.dfrNoComment(t_textA)
      if t_textB==nil
        return FAILED
      end
      t_textA=t_textB
      @h_BreakpointHash.store(t_textA,t_textA)
      @i_idx+=1
      i_bpindex+=1
    end
    #i_return=self.dfrShowHashOnRubyConsole(@h_BreakpointHash,'Breakpoints')
# why didn't this work properly? 
  end
  return SUCCESS
rescue
   return FAILED
#end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrGetWatchesFromSetupFile
#
# This method gets watch information from the setup file
#
begin
#
# stash count of watches
#
  t_textA=@ts_setup[@i_idx]
  @i_idx+=1
  i_watchcount=t_textA.to_i
#
#  fill hash for watched variables and conditions
#
  if i_watchcount>0
    @h_WatchValueHash={}
    @h_WatchCodeHash={}
    i_add=@i_idx
    i_seqnumber=0
    while @i_idx<i_watchcount+i_add
      i_seqnumber+=1
      t_textA=@ts_setup[@i_idx].strip
      t_textB=Debuggifier.dfrNoComment(t_textA)
      if t_textB==nil
        return FAILED
      end
      t_textA=t_textB
#      self.dfrUnphrase(t_textA)
#      ts_words=t_textA.scan(/\w+/)
      ts_words=t_textA.split(' ',3)
      t_varname=ts_words[0]
      if t_varname.length==0
        UI.messagebox('Empty variable specifier, assuming <count>')
        t_condstring='CHANGED'
      end
      t_value=ts_words[2].to_s
      if t_value.length==0
        t_value='0'
      end
      t_condstring=ts_words[1].to_s
      if t_condstring.length==0
        UI.messagebox('Empty watch condition, assuming <CHANGED>')
        t_condstring='CHANGED'
        @i_ConditionExprType=COND_CHANGED
      else
        @i_ConditionExprType=self.dfrGetNumericConditionCode(t_condstring)
      end
      t_first=t_value[0]
      i_first=t_first.to_i
      if i_first==39  #apostrophe?
        i_Len=t_value.length
        u_value=t_value[1,i_Len-1]
      else
        u_value=t_value.to_f
      end
      @h_WatchValueHash.store(t_varname,u_value)
      @h_WatchCodeHash.store(t_varname,@i_ConditionExprType)
      @i_idx+=1
    end
    #i_return=self.dfrShowHashOnRubyConsole(@h_WatchValueHash,'Watches')
  end
  return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrUnphrase(t_text)
#
# This method implements .scan the way it should be implemented.  The Ruby
# version is thrown off by characters like ">" and "."
# So what if it is slow.  It is only invoked when speed does not matter...
# Note that doublespaces will not be handled properly, only
# singlespaces will be handled properly
#
begin
  @t_phrase0=''
  @t_phrase1=''
  @t_phrase2=''
  t_work=t_text.strip
  i_Len=t_work.length
  if i_Len<5
    return FAILED
  end
  @t_build=''
  i_idx=0
  i_phrase=0
  while i_idx<i_Len
    i_char=t_work[i_idx]
    t_char
    if t_char==' '
      if i_phrase==0
        @t_phrase0=@t_build
        @t_build=''
      elsif i_phrase==1
        @t_phrase1=@t_build
        @t_build=''
      else
        @t_phase2=@t_build #should never get to this statement
      end
      i_phrase+=1
    else

      t_new=@t_build+t_char     
      @t_build=t_new
    end
    i_idx+=1
  end
  @t_phase2=@t_build
  return SUCCESS
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
def Debuggifier.dfrNoComment(t_text)
#
#  This method removes any inline comment denoted by '#' from a string
#
begin
  t_work=t_text.to_s.strip
  i_found=t_work.index('#')
  if i_found!=nil
    t_text=t_work[0,i_found-1].strip
  end
  return t_text
rescue
  return FAILED
end
end
#----------------------------------------------------------------------------
end #class Debuggifier
#----------------------------------------------------------------------------
#
# Saved code relevant to interacting with DebugView
#
#getCursorPos = Win32API.new("user32", "GetCursorPos", ['P'], 'V')
#lpPoint = " " * 8 # store two LONGs
#getCursorPos.Call(lpPoint)
#x, y = lpPoint.unpack("LL") # get the actual values
#print "x: ", x, "\n"
#print "y: ", y, "\n"
#ods = Win32API.new("kernel32", "OutputDebugString", ['P'], 'V')
#ods.Call("Hello, World\n")
#GetDesktopWindow = Win32API.new("user32", "GetDesktopWindow", [], 'L')
#GetActiveWindow = Win32API.new("user32", "GetActiveWindow", [], 'L')
#SendMessage = Win32API.new("user32", "SendMessage", ['L'] * 4, 'L')
#SendMessage.Call(GetDesktopWindow.Call, 274, 0xf140, 0)
